#!/usr/bin/env python3

import argparse
from flask import Flask, render_template, request, redirect, url_for, send_from_directory
import os
import urllib.parse
import socket
import logging
import subprocess
import re
import sys
from jinja2 import ChoiceLoader, FileSystemLoader, PackageLoader

# About : inspired by updog we named it updogfx Date 18/05/2025
# Supports most Linux-based OS with SSH and Bash installed
# Version 2.0

# Suppress Flask and Werkzeug logs
log = logging.getLogger('werkzeug')
log.setLevel(logging.CRITICAL)

cli = logging.getLogger('flask.cli')
cli.setLevel(logging.CRITICAL)

# Base directory of this app.py file
basedir = os.path.abspath(os.path.dirname(__file__))

# Setup Flask app WITHOUT setting template_folder here (let Jinja load it)
app = Flask(__name__)

# Configure Jinja2 loaders to support both local templates and package templates
app.jinja_loader = ChoiceLoader([
    FileSystemLoader(os.path.join(basedir, "templates")),  # Local templates for development
    PackageLoader('updogfx', 'templates'),                  # Package templates for installed mode
])

UPLOAD_FOLDER = os.getcwd()
app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER

def parse_arguments():
    parser = argparse.ArgumentParser(description='Simple file upload server with options.')
    parser.add_argument('-d', '--directory', default=UPLOAD_FOLDER,
                        help='Directory to store uploaded files (default: current directory)')
    return parser.parse_args()

def setup_directory(directory):
    if not os.path.exists(directory):
        os.makedirs(directory)
        flush_print(f"[+] Upload directory created: {directory}")

def save_file(file, filename):
    file.save(os.path.join(app.config['UPLOAD_FOLDER'], filename))

@app.route('/')
def index():
    files = os.listdir(app.config['UPLOAD_FOLDER'])
    return render_template('index.html', files=files)

@app.route('/upload', methods=['POST'])
def upload_file():
    if 'file' not in request.files or request.files['file'].filename == '':
        return redirect(request.url)
    file = request.files['file']
    filename = urllib.parse.quote(file.filename)
    save_file(file, filename)
    flush_print(f"[+] File uploaded: {urllib.parse.unquote(filename)}")
    return redirect(url_for('index'))

@app.route('/uploads/<path:filename>')
def uploaded_file(filename):
    filename = urllib.parse.unquote(filename)
    flush_print(f"[+] File downloaded: {filename}")
    return send_from_directory(app.config['UPLOAD_FOLDER'], filename)

def linkgens():
    try:
        process = subprocess.Popen(
            ['ssh', '-o', 'StrictHostKeyChecking=no', '-R', '80:127.0.0.1:8080', 'serveo.net'],
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            text=True
        )
        while True:
            output = process.stdout.readline()
            if output == '' and process.poll() is not None:
                break
            if output:
                match = re.search(r'https://[a-zA-Z0-9]+\.serveo\.net', output)
                if match:
                    flush_print(f"Upload on: {match.group(0)}")
                    break
    except Exception as e:
        flush_print(f"Error: {e}")

def flush_print(message):
    print(message, flush=True)

def suppress_flask_logs():
    sys.stdout = open(os.devnull, 'w')
    sys.stderr = open(os.devnull, 'w')

def main():
    args = parse_arguments()
    app.config['UPLOAD_FOLDER'] = args.directory
    setup_directory(app.config['UPLOAD_FOLDER'])

    try:
        local_ip = socket.gethostbyname(socket.gethostname())
    except socket.gaierror:
        local_ip = "127.0.0.1"

    port = 8080
    flush_print(f"\nUpload on: http://{local_ip}:{port}")

    linkgens()

    suppress_flask_logs()

    app.run(host='0.0.0.0', port=port, debug=True, use_reloader=False, threaded=True)

if __name__ == '__main__':
    main()

